@@ -9,7 +9,7 @@ use base 'Class::MOP::Class';
 sub mymetaclass_attributes{
   my $self = shift;
   return grep { $_->isa("MyMetaClass::Attribute") }
-    $self->compute_all_applicable_attributes;
+    $self->get_all_attributes;
 }
 
 1;
